#ifdef _MSC_VER
#define _CRT_SECURE_NO_DEPRECATE
#endif

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#if defined(__GNUC__) && !defined(__CYGWIN__)
#include <unistd.h>
#endif

#if SPLIT == 0
#include "zsfmt.c"
#else
#include "zsfmt.h"
#endif

#ifndef MSEQ
#define MSEQ 1
#endif

#if (defined(__TURBOC__)&&!defined(__BORLANDC__))|| defined(LSI_C)
#define T 10000000L
#define U 100000000L
#define V 1000000L
#else
#define T 100000000
#define U 100000000
#define V 10000000
#endif

#ifdef LSI_C
#undef CLOCKS_PER_SEC
#endif

#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC 1000
#include <dos.h>
static long clock(void) {
    union REGS t; t.h.ah=0x2c; intdos(&t,&t);
    return t.h.dl*10L+t.h.dh*1000L+t.h.cl*60000L+t.h.ch*3600000L;
}
#endif

FILE*fp;
char fname[1024];
int Mode;

char*StrUpr(char*s) {
    char*p=s;
    while (*p) { if (*p>='a'&&*p<='z') *p+='A'-'a'; p++; }
    return s;
}

void PutStr(char*s)     { fprintf(fp,"%s",s); }
void PutDbl(double x)   { fprintf(fp," %11.7f",x); }
void PutInt(int32_t x)  { fprintf(fp," %3ld",(long)x); }
void PutUsg(uint32_t x) { fprintf(fp,"%10lu ",(unsigned long)x); }
void PutBit(int x)      { fprintf(fp,"%d",x); };
void PutLn1(char*s)     { fprintf(fp,"\r\n%-9s",s); }
void PutLn2(char*s)     { fprintf(fp,"%s\r\n",s); }

void TestDistBitByte(int m) {
    int i;
    PutLn1("Bit");
    for (i=0;i<48;i++) fprintf(fp,"%d",NextBit_m(m));
    PutLn1("Byte");
    for (i=0;i<12;i++) fprintf(fp," %3d",NextByte_m(m));
}

void EachDist(int m)
{
    int i; static double vec[4];
    static uint32_t ini[4]={0x1234,0x5678,0x9abc,0xdef0};

    for (i=0;i<10000;i++) NextMt_m(m);
    PutStr(IdString_m(m));
    PutLn1("Unif");
    for (i=0;i<4;i++) PutDbl(NextUnif_m(m));
    PutLn1("Int");
    for (i=0;i<12;i++) PutInt(NextInt_m(m,1000));
    PutLn1("IntEx");
    for (i=0;i<12;i++) PutInt(NextIntEx_m(m,1000));
    PutLn1("Chisq");
    for (i=0;i<4;i++) PutDbl(NextChisq_m(m,10));
    PutLn1("Gamma");
    for (i=0;i<4;i++) PutDbl(NextGamma_m(m,10));
    PutLn1("Geometric");
    for (i=0;i<12;i++) PutInt(NextGeometric_m(m,0.2));
    PutLn1("Triangle");
    for (i=0;i<4;i++) PutDbl(NextTriangle_m(m));
    PutLn1("Exp");
    for (i=0;i<4;i++) PutDbl(NextExp_m(m));
    PutLn1("Normal");
    for (i=0;i<4;i++) PutDbl(NextNormal_m(m));
    PutLn1("UnitVect"); NextUnitVect_m(m,vec,4);
    for (i=0;i<4;i++) PutDbl(vec[i]);
    PutLn1("Binomial");
    for (i=0;i<12;i++) PutInt(NextBinomial_m(m,10,0.3));
    PutLn1("Binormal");
    for (i=0;i<4;i+=2) NextBinormal_m(m,0.3,vec+i,vec+i+1);
    for (i=0;i<4;i++) PutDbl(vec[i]);
    PutLn1("Beta");
    for (i=0;i<4;i++) PutDbl(NextBeta_m(m,2.0,3.0));
    PutLn1("Power");
    for (i=0;i<4;i++) PutDbl(NextPower_m(m,10));
    PutLn1("Logistic");
    for (i=0;i<4;i++) PutDbl(NextLogistic_m(m));
    PutLn1("Cauchy");
    for (i=0;i<4;i++) PutDbl(NextCauchy_m(m));
    PutLn1("FDist");
    for (i=0;i<4;i++) PutDbl(NextFDist_m(m,10,20));
    PutLn1("Poisson");
    for (i=0;i<12;i++) PutInt(NextPoisson_m(m,2.0));
    PutLn1("TDist");
    for (i=0;i<4;i++) PutDbl(NextTDist_m(m,10));
    PutLn1("Weibull");
    for (i=0;i<4;i++) PutDbl(NextWeibull_m(m,0.3));
    TestDistBitByte(m);
    PutLn1("Init"); InitMt_m(m,1234);
    for (i=0;i<4;i++) PutDbl(NextUnif_m(m));
    PutLn1("InitEx"); InitMtEx_m(m,ini,4);
    for (i=0;i<4;i++) PutDbl(NextUnif_m(m));
    PutLn2("");
}

void TestDist(void)
{
    int m;

    sprintf(fname,"CD%ld.TXT",(long)MEXP);
    fp=fopen(fname,"wb");
    for (m=0;m<(MSEQ2>MSEQ?MSEQ2:MSEQ);m++)  EachDist(m);
    fclose(fp);
    printf("'%s'\n",fname);
}

void TestOut(void)
{       
    uint32_t ini[]={0x1234, 0x5678, 0x9abc, 0xdef0}; int i;

    sprintf(fname,"CO%ld.TXT",(long)MEXP);
    fp=fopen(fname,"wb");
    PutLn2(IdString);

    InitMt(1234);
    PutLn2("32 bit generated randoms");
    PutLn2("init_gen_rand__________");
    for (i=0;i<1000;i++) {
        PutUsg(NextMt());
        if (i%5==4) PutLn2("");
    }
    PutLn2("");

    InitMtEx(ini,4);
    PutLn2("init_by_array__________");
    for (i=0;i<1000;i++) {
        PutUsg(NextMt());
        if (i%5==4) PutLn2("");
    }

    fclose(fp);
    printf("'%s'\n",fname);
}

#if defined(LSI_C)
#define COMP "LCC     "
#elif defined(__BORLANDC__)
#define COMP "BCC     "
#elif defined(__TURBOC__)
#define COMP "TCC     "
#elif defined(__GNUC__) && defined(__CYGWIN__)
#define COMP "GCC-4   "
#elif defined(__GNUC__) && !defined(__CYGWIN__) && __WORDSIZE == 32
#define COMP "GCC-32  "
#elif defined(__GNUC__) && !defined(__CYGWIN__) && __WORDSIZE == 64
#define COMP "GCC-64  "
#elif defined(_MSC_VER) && _MSC_VER == 1200
#define COMP "VC++60  "
#elif defined(_MSC_VER) && _MSC_VER == 1300
#define COMP "VC++2002"
#elif defined(_MSC_VER) && _MSC_VER == 1310
#define COMP "VC++2003"
#elif defined(_MSC_VER) && _MSC_VER == 1400
#define COMP "VC++2005"
#elif defined(_MSC_VER) && _MSC_VER == 1500
#define COMP "VC++2008"
#else
#define COMP "UNKNOWN"
#endif

#ifdef STABLE_SPEED
#define SPEED "STABLE"
#else
#define SPEED "LOOP  "
#endif

uint32_t IntLoop(void)
{
    int32_t i; uint32_t sum=0;

    for (i=0;i<T;i++) sum+=NextMt();
    return sum;
}

uint32_t BitLoop(void)
{
    int32_t i; uint32_t sum=0;

    for (i=0;i<U;i++) sum+=NextBit();
    return sum;
}

double UnifLoop(void)
{
    int32_t i; double sum=0;

    for (i=0;i<V;i++) sum+=NextUnif();
    return sum;
}

void TestSpeed(char*sse)
{       
    long clo,min=1048576L; double rst; int k; char*p;
    static uint32_t IntSum,BitSum; static double DblSum;

    if (Mode) {
#if defined(__GNUC__) && !defined(__CYGWIN__)
        static char buf[256];
        gethostname(buf,sizeof buf); p=StrUpr(buf);
#else    
        p=StrUpr(getenv("USERDOMAIN"));
#endif
        if (p==0) p="UNKNOWN";
        sprintf(fname,"../%.8s.TXT",p);
        fp=fopen(fname,"ab");
    }
    printf("%s\n",sse);
    printf("U32  ");
    for (k=0;k<10;k++) {
        clo=clock();
        IntSum+=IntLoop();
        clo=clock()-clo;
        printf(" %ld",clo*1000/(long)CLOCKS_PER_SEC);
        fflush(stdout);
        if (clo<min) min=clo;
    }
    rst=100000000.0*min/CLOCKS_PER_SEC/T;
    printf("(%.3f)\n",rst);
    if (Mode)
        fprintf(fp,"zsfmt %s MEXP=%ld %s %s Int  %8.3f\r\n",
             SPEED,(long)MEXP,COMP,sse,rst);
    min=1048576L;
    printf("Bit  ");
    for (k=0;k<10;k++) {
        clo=clock();
        BitSum+=BitLoop();
        clo=clock()-clo;
        printf(" %ld",clo*1000/(long)CLOCKS_PER_SEC);
        fflush(stdout);
        if (clo<min) min=clo;
    }
    rst=100000000.0*min/CLOCKS_PER_SEC/U;
    printf("(%.3f)\n",rst);
    if (Mode)
        fprintf(fp,"zsfmt %s MEXP=%ld %s %s Bit  %8.3f\r\n",
             SPEED,(long)MEXP,COMP,sse,rst);
    min=1048576L;
    printf("Unif ");
    for (k=0;k<10;k++) {
        clo=clock();
        DblSum+=UnifLoop();
        clo=clock()-clo;
        printf(" %ld",clo*1000/(long)CLOCKS_PER_SEC);
        fflush(stdout);
        if (clo<min) min=clo;
    }
    rst=100000000.0*min/CLOCKS_PER_SEC/V;
    printf("(%.3f)\n",rst);
    if (Mode)
        fprintf(fp,"zsfmt %s MEXP=%ld %s %s Unif %8.3f\r\n",
             SPEED,(long)MEXP,COMP,sse,rst);
    if (Mode) fclose(fp);
}

int main(int argc,char**argv) {
    Mode=(argc==2&&argv[1][0]=='-');
    printf("%s zsfmt\n",COMP);
    if (Mode==0) { TestDist(); TestOut(); }
    InitMt(1234);
    if (DefaultMt->flag==1)
    { TestSpeed("SSE2"); DefaultMt->flag=0; }
    TestSpeed("i386");
    return 0;
}
